// evCmdLineShim.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "evCmdLineShim.h"

// Global Variables
TERMTOINPUTMAP terminalsMap;
int width = 392;
int height = 377;
int pre = 5;
int post = 5;

#define ODS_WITH_MSG_BOX

//---------------------------------------------------------------------------
int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
    // Parse command line for terminal number
    int searchTerm = 101;
    std::string cmdLine = std::string(lpCmdLine);
    string::size_type termPos = cmdLine.find("-n");
    if (termPos == string::npos) termPos = cmdLine.find("-N");
    if (termPos == string::npos) termPos = cmdLine.find("/n");
    if (termPos == string::npos) termPos = cmdLine.find("/N");
    if (termPos != string::npos)
        searchTerm = atoi(cmdLine.substr(termPos+2,cmdLine.find_first_of(' ',termPos+2)).c_str());

    // Parse command line for search time - defaults to current time if not present
    time_t searchTime_t;
#ifdef USE_DOT_NET
    // If using .NET for datetime parsing, set /clr option (use managed extensions) in VC project file 
    DateTime searchDateTime = DateTime::get_Now();  // default to current time if no time set on cmd line
#else
    time(&searchTime_t);    // default to current time if no time set on cmd line
#endif
    string::size_type timePos = cmdLine.find("-t");
    if (timePos == string::npos) termPos = cmdLine.find("-T");
    if (timePos == string::npos) termPos = cmdLine.find("/t");
    if (timePos == string::npos) termPos = cmdLine.find("/T");
    if (timePos != string::npos) {
        string::size_type timeEndPos = cmdLine.find_first_of("-n",timePos+2);
        if (timeEndPos == string::npos) timeEndPos = cmdLine.find_first_of("-N",timePos+2);
        if (timeEndPos == string::npos) timeEndPos = cmdLine.find_first_of("/n",timePos+2);
        if (timeEndPos == string::npos) timeEndPos = cmdLine.find_first_of("/N",timePos+2);
#ifdef USE_DOT_NET
        System::String *dateTimeStr = cmdLine.substr(timePos+2, timePos+2+timeEndPos).c_str();
        searchDateTime = DateTime::Parse(dateTimeStr);
    }
    // exacqVision search time is in time_t (number of seconds since
    // Jan 1, 1970 00:00:00 UTC) so we derive that from the input format
    TimeSpan utcOffset = DateTime::get_UtcNow() - DateTime::get_Now();
    DateTime startTime = DateTime(1970,1,1);
    TimeSpan tsSearchTime_t = searchDateTime - startTime + utcOffset;
    double fSearchTime_t = tsSearchTime_t.TotalSeconds;
    searchTime_t = (time_t)fSearchTime_t;
#else
        std::string dateTimeStr = cmdLine.substr(timePos+2, timePos+2+timeEndPos);
        struct tm cmdLineTm;
        memset(&cmdLineTm, 0, sizeof(tm));
        // Command Line Time/Date format is:  2007-06-29 15:51:05
        strptime(dateTimeStr.c_str(), "%Y-%m-%d %T", &cmdLineTm);
        cmdLineTm.tm_isdst = -1; // let crt figure out whether DST is in effect and apply it to returned time_t
        searchTime_t = mktime(&cmdLineTm);
    }
#endif

    // Now read the XML which maps terminal to exacqVision input
    if (!ParseConfigXML("termToInput.xml")) {
        // TODO - default term to input mapping
        return 0;
    }

    // Now write exacqVision command line XML
    if (!WriteCmdLineXML("C:\\Program Files\\exacqVision\\Client\\eVCmdLine.xml", searchTerm, searchTime_t)) {
        return 0;
    }


    // Now spawn exacqVision Client with command line input
    /* CreateProcess API initialization */
    STARTUPINFO siStartupInfo;
    PROCESS_INFORMATION piProcessInfo;
    memset(&siStartupInfo, 0, sizeof(siStartupInfo));
    memset(&piProcessInfo, 0, sizeof(piProcessInfo));
    siStartupInfo.cb = sizeof(siStartupInfo);
    /* Execute */
    CreateProcess(NULL, 
        "C:\\Program Files\\exacqVision\\Client\\edvrclient.exe -FeVCmdLine.xml",
        0, 0, false,
        NORMAL_PRIORITY_CLASS, 0, 
        "C:\\Program Files\\exacqVision\\Client",   // set eV Client working directory
        &siStartupInfo,
        &piProcessInfo);
    CloseHandle( piProcessInfo.hProcess );
    CloseHandle( piProcessInfo.hThread );
    return 0;
}
//---------------------------------------------------------------------------
int ParseConfigXML(const char *configFileName)
{
    bool foundTermMapping = false;
    std::ifstream file;
    file.open(configFileName);
    if (!file) {
        ODS("Could not open file %s\n",configFileName);
        return false;
    }

    int length = 0;
    while(file.get() != std::ios::traits_type::eof())
        ++length;
    file.clear();
    file.seekg(0);

    char* pXML = (char*)malloc(length);
    file.read(pXML, length);

    // Parse XML command line config file.
    TiXmlDocument doc;
    string str(pXML, length);
    istringstream stm(str);
    stm >> doc;
    file.close();
    if (doc.Error()) {
        ODS("Cmd line configuration document XML error: %s", doc.ErrorDesc());
        return false;
    }

    // Parse Command Line File.
    TiXmlElement* rootNode = doc.FirstChildElement("TermToInput");
    if ( rootNode ) {
        int temp;
        bool valid = true;

        if (TIXML_SUCCESS == rootNode->QueryIntAttribute("Width", &temp))
            width = temp;
        if (TIXML_SUCCESS == rootNode->QueryIntAttribute("Height", &temp))
            height = temp;
        if (TIXML_SUCCESS == rootNode->QueryIntAttribute("Pre", &temp))
            pre = temp;
        if (TIXML_SUCCESS == rootNode->QueryIntAttribute("Post", &temp))
            post = temp;
        // Parse terminal to input data
        for (TiXmlElement* elemTerm=rootNode->FirstChildElement("Term"); elemTerm; elemTerm=elemTerm->NextSiblingElement("Term")) 
        {
            int termNum;
            InputInfo term;
            if (TIXML_SUCCESS == elemTerm->QueryIntAttribute("Number", &termNum)) {
                TiXmlElement* elemSystem=elemTerm->FirstChildElement("System");
                if (elemSystem) {
                    const std::string *sysName = elemSystem->Attribute(std::string("Name"));
                    if (sysName) {
                        TiXmlElement* elemInput=elemSystem->FirstChildElement("Input");
                        if (elemInput)
                        {
                            const std::string *inputName = elemInput->Attribute(std::string("Name"));
                            const std::string *inputNum = elemInput->Attribute(std::string("Number"));
                            const std::string *devAddr = elemInput->Attribute(std::string("Device"));
                            term.system = *sysName;
                            if (inputName)
                                term.name = *inputName;
                            if (inputNum)
                                term.number = *inputNum;
                            if (devAddr)
                                term.devAddr = *devAddr;
                            if (inputName || inputNum) {
                                terminalsMap[termNum] = term;
                                foundTermMapping = true;
                            }
                            else {
                                ODS("Found invalid mapping for Term Number %d in file %s\n",termNum,configFileName);
                            }
                        }
                    }
                }
            }
        }
    }
    if (pXML)
        free(pXML);
    return foundTermMapping;
}
//---------------------------------------------------------------------------
bool WriteCmdLineXML(const char *fileName, int searchTerm, time_t searchTime)
{
    bool found = false, isOK = false;
    POINT pos;
    GetCursorPos(&pos);
    TiXmlDocument doc;
    TiXmlDeclaration * decl = new TiXmlDeclaration( "1.0","","" );
    doc.LinkEndChild(decl);

    TiXmlElement *rootTi = new TiXmlElement("exacqVisionInit");
    doc.LinkEndChild(rootTi);

    rootTi->SetAttribute("Top", pos.y);
    rootTi->SetAttribute("Left", pos.x);
    rootTi->SetAttribute("Width", width);
    rootTi->SetAttribute("Height", height);
    rootTi->SetAttribute("Maximize", "0");
    rootTi->SetAttribute("ShowNavPanel", "0");
    rootTi->SetAttribute("ShowToolbars", "0");
    rootTi->SetAttribute("FullScreen", "0");
    rootTi->SetAttribute("AllowAccelerators", "0");
    TiXmlElement *SearchTiE = getTiXmlElement(rootTi, "Search");
    SearchTiE->SetAttribute("Start", (int)searchTime - pre);
    SearchTiE->SetAttribute("End", (int)searchTime + post);
    TERMTOINPUTMAP::iterator t_iter;
    t_iter = terminalsMap.find(searchTerm);
    if (t_iter != terminalsMap.end()) {
        InputInfo searchInput = (*t_iter).second;
        TiXmlElement *systemTiE = getTiXmlElement(SearchTiE, "System", "Name", searchInput.system);
        TiXmlElement *videoTiE = getTiXmlElement(systemTiE, "Video");
        if (!searchInput.name.empty())
            getTiXmlElement(videoTiE, "Input", "Name", searchInput.name);
        else {
            TiXmlElement *inputTiE = getTiXmlElement(videoTiE, "Input", "Number", searchInput.number);
            if (!searchInput.devAddr.empty())  // IP cameras must specify address, eDVR board inputs don't
                inputTiE->SetAttribute("Device", searchInput.devAddr);
        }
        found = true;
    }
    else 
        ODS("Mapping of search terminal %d to camera not found.\n", searchTerm);
    if (found) {
        isOK = doc.SaveFile(fileName);
        if (!isOK)
            ODS("Error writing file %s.  Error %s\n",doc.ErrorDesc());
    }
    return isOK;
}
//---------------------------------------------------------------------------
void ODS(LPCTSTR lpOutputString, ...)
{
    char achBuffer[256];
    va_list args;

    va_start(args, lpOutputString);
    vsprintf(achBuffer, lpOutputString, args);
    va_end(args);
    if (achBuffer[strlen(achBuffer)-1] != '\n') strcat(achBuffer, "\n");
#if defined _DEBUG && !defined ODS_WITH_MSG_BOX
    OutputDebugString(achBuffer);
#else
    MessageBox(GetDesktopWindow(), achBuffer, "POS Audit to Video Interface", MB_OK);
#endif
}
//---------------------------------------------------------------------------
TiXmlElement * getTiXmlElement(TiXmlElement* level, const std::string &node, const std::string &attr, const std::string &val)
{
    TiXmlElement *rv = NULL;
    TiXmlElement *temp = level->FirstChildElement(node);
    while(temp != NULL) {
        const std::string *tempStr = temp->Attribute(attr);
        if (tempStr) {
            if(val == *tempStr) {
                rv = temp;
                break;
            }
        }
        temp = temp->NextSiblingElement(node);
    }
    if(rv == NULL) {
        rv = new TiXmlElement(node);
        rv->SetAttribute(attr, val);
        level->LinkEndChild(rv);
    }
    return rv;
}
//---------------------------------------------------------------------------
TiXmlElement * getTiXmlElement(TiXmlElement* level, const std::string &node)
{
    TiXmlElement *rv = level->FirstChildElement(node);
    if(rv == NULL) {
        rv = new TiXmlElement(node);
        level->LinkEndChild(rv);
    }
    return rv;
}


